<?php

namespace OzzModz\WebsiteVerification\Api\Controller;

use XF\Mvc\Entity\Entity;
use XF\Mvc\ParameterBag;

class Websites extends \XF\Api\Controller\AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertApiScopeByRequestMethod('ozzmodz_wv_website');
	}

	/**
	 * @api-desc Creates a new website.
	 *
	 * @api-in str $domain <req>
	 *
	 * @api-out true $success
	 * @api-out OzzModz_WebsiteVerification_Website $website
	 *
	 * @return \XF\Api\Mvc\Reply\ApiResult|\XF\Mvc\Reply\Error
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionPost()
	{
		$this->assertRequiredApiInput('domain');
		$this->assertRegisteredUser();

		if (\XF::isApiCheckingPermissions())
		{
			/** @var \OzzModz\WebsiteVerification\XF\Entity\User $visitor */
			$visitor = \XF::visitor();
			if (!$visitor->canAddWebsiteVerificationDomains($error))
			{
				throw $this->exception($this->noPermission($error));
			}
		}

		$creator = $this->setupWebsiteCreate();
		if (!$creator->validate($errors))
		{
			return $this->error($errors);
		}

		/** @var \OzzModz\WebsiteVerification\Entity\Website $website */
		$website = $creator->save();
		$this->finalizeWebsiteCreate($creator);

		return $this->apiSuccess([
			'website' => $website->toApiResult(Entity::VERBOSITY_VERBOSE)
		]);
	}

	protected function setupWebsiteCreate()
	{
		/** @var \OzzModz\WebsiteVerification\Service\Website\Create $creator */
		$creator = $this->service('OzzModz\WebsiteVerification:Website\Create');
		$creator->setDomain($this->filter('domain', 'str'));

		return $creator;
	}

	protected function finalizeWebsiteCreate(\OzzModz\WebsiteVerification\Service\Website\Create $creator)
	{
	}

	/**
	 * @api-desc Runs verification check specified website
	 *
	 * @api-in int $website_id <req>
	 * @api-in str $xenforo_validation_token
	 *
	 * @api-out true $success
	 * @api-out OzzModz_WebsiteVerification_Website $website
	 *
	 * @return \XF\Api\Mvc\Reply\ApiResult|\XF\Mvc\Reply\Error
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionPostVerify(ParameterBag $params)
	{
		$website = $this->assertViewableWebsite($params->website_id);

		if (\XF::isApiCheckingPermissions())
		{
			if (!$website->canEdit($error))
			{
				throw $this->exception($this->noPermission($error));
			}
		}

		/** @var \OzzModz\WebsiteVerification\Service\Website\Verify $verifier */
		$verifier = $this->service('OzzModz\WebsiteVerification:Website\Verify', $website);

		$validationToken = $this->filter('xenforo_validation_token', 'str');
		if ($validationToken)
		{
			$verifier->setXenforoValidationToken($validationToken);
		}

		if (!$verifier->validate($errors))
		{
			return $this->error($errors);
		}

		return $this->apiSuccess([
			'website' => $website->toApiResult(Entity::VERBOSITY_VERBOSE)
		]);
	}

	/**
	 * @param $id
	 * @param $with
	 * @return Entity|\OzzModz\WebsiteVerification\Entity\Website
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertViewableWebsite($id, $with = 'api')
	{
		return $this->assertViewableApiRecord('OzzModz\WebsiteVerification:Website', $id, $with);
	}
}