<?php

namespace OzzModz\WebsiteVerification\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;
use function Aws\filter;

/**
 * COLUMNS
 * @property int $website_id
 * @property string $domain
 * @property int $user_id
 * @property string $verification_code
 * @property string $xenforo_validation_token
 * @property int $date_added
 * @property int $validation_date
 * @property int $last_update
 * @property bool $dns_verified
 * @property bool $file_verified
 *
 * GETTERS
 * @property bool $is_verified
 * @property string $txt_record
 *
 * RELATIONS
 * @property \OzzModz\WebsiteVerification\XF\Entity\User $User
 * @property XenforoLicense $XenforoLicense
 */
class Website extends Entity
{
	public function canView(&$error = null)
	{
		/** @var \OzzModz\WebsiteVerification\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		if ($visitor->hasPermission('ozzmodz_wv', 'viewAny'))
		{
			return true;
		}

		return $visitor->user_id == $this->user_id && $visitor->hasPermission('ozzmodz_wv', 'viewOwn');
	}

	public function canEdit(&$error = null)
	{
		/** @var \OzzModz\WebsiteVerification\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		return $visitor->user_id == $this->user_id;
	}

	public function canDelete(&$error = null)
	{
		/** @var \OzzModz\WebsiteVerification\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		return $visitor->user_id == $this->user_id;
	}

	public function isVerified()
	{
		return $this->dns_verified || $this->file_verified || ($this->XenforoLicense && $this->XenforoLicense->is_valid);
	}

	public function getTxtRecord()
	{
		return $this->app()->options()->ozzmodz_wv_verificationKey . ':' . $this->verification_code;
	}

	public function getVerificationFilePath()
	{
		return "$this->domain/{$this->app()->options()->ozzmodz_wv_verificationKey}.txt";
	}

	/**
	 * @param \XF\Api\Result\EntityResult $result
	 * @param $verbosity
	 * @param array $options
	 * @return void
	 *
	 * @api-out bool $can_view
	 * @api-out bool $can_delete
	 * @api-out string $txt_record
	 * @api-out string $verification_file_path
	 * @api-out bool $is_verified
	 *
	 * @noinspection PhpMissingParamTypeInspection
	 */
	protected function setupApiResultData(\XF\Api\Result\EntityResult $result, $verbosity = self::VERBOSITY_NORMAL, array $options = [])
	{
		$result->can_view = $this->canView();
		$result->can_delete = $this->canDelete();

		$result->txt_record = $this->txt_record;
		$result->verification_file_path = $this->getVerificationFilePath();
		$result->is_verified = $this->is_verified;

		if (!empty($options['with_xenforo_license']))
		{
			$result->includeRelation('XenforoLicense');
		}
	}

	// ################################## LIFECYCLE ###########################################

	protected function _preSave()
	{
		if (!filter_var($this->domain, FILTER_VALIDATE_DOMAIN, FILTER_FLAG_HOSTNAME))
		{
			$this->error(\XF::phrase('ozzmodz_wv_please_enter_valid_domain'), 'domain');
			return;
		}

		$fakeUrl = 'http://' . str_replace(['http://', 'https://'], '', $this->domain);
		$hostname = parse_url($fakeUrl, PHP_URL_HOST);
		if (!$hostname)
		{
			$this->error(\XF::phrase('ozzmodz_wv_please_enter_valid_domain'), 'domain');
			return;
		}

		$this->domain = $hostname;
	}

	protected function _postSave()
	{
		$user = $this->User;
		if ($user && $this->isInsert() || $this->isChanged(['dns_verified', 'file_verified']))
		{
			$user->rebuildOzzModzWebsiteVerificationCounters();
			$user->save();
		}
	}

	protected function _postDelete()
	{
		$user = $this->User;
		if ($user)
		{
			$user->rebuildOzzModzWebsiteVerificationCounters();
			$user->save();
		}
	}

	/**
	 * @param Structure $structure
	 * @return Structure
	 */
	public static function getStructure(Structure $structure)
	{
		$structure->table = 'xf_ozzmodz_wv_website';
		$structure->shortName = 'OzzModz\WebsiteVerification:Website';
		$structure->primaryKey = 'website_id';
		$structure->contentType = 'ozzmodz_wv_website';
		$structure->columns = [
			'website_id' => ['type' => static::UINT, 'autoIncrement' => true],
			'domain' => ['type' => static::STR, 'maxLength' => 255, 'required' => true, 'api' => true],
			'user_id' => ['type' => static::UINT, 'required' => true],
			'verification_code' => ['type' => static::BINARY, 'maxLength' => 32, 'required' => true, 'api' => true],
			'xenforo_validation_token' => ['type' => static::STR, 'maxLength' => 50, 'default' => '', 'api' => true],
			'date_added' => ['type' => static::UINT, 'default' => \XF::$time, 'api' => true],
			'validation_date' => ['type' => static::UINT, 'default' => '0', 'api' => true],
			'last_update' => ['type' => static::UINT, 'default' => '0', 'api' => true],
			'dns_verified' => ['type' => static::BOOL, 'default' => false, 'api' => true],
			'file_verified' => ['type' => static::BOOL, 'default' => false, 'api' => true]
		];

		$structure->relations = [
			'User' => [
				'entity' => 'XF:User',
				'type' => self::TO_ONE,
				'conditions' => 'user_id',
				'primary' => true
			],
			'XenforoLicense' => [
				'entity' => 'OzzModz\WebsiteVerification:XenforoLicense',
				'type' => self::TO_ONE,
				'conditions' => [
					['validation_token', '=', '$xenforo_validation_token']
				],
				'primary' => true
			]
		];

		$structure->withAliases = [
			'api' => [
				'User',
				'User.api',
				function($withParams)
				{
					if (!empty($withParams['xenforo_license']))
					{
						return ['XenforoLicense.api'];
					}
				}
			]
		];

		$structure->getters = [
			'txt_record' => true,
			'is_verified' => ['getter' => 'isVerified']
		];

		return $structure;
	}

	public function resetErrors($keys = [])
	{
		if (!$keys)
		{
			$this->_errors = [];
		}
		else
		{
			if (!is_array($keys))
			{
				$keys = [$keys];
			}

			foreach ($keys as $key)
			{
				unset($this->_errors[$key]);
			}
		}
	}
}
