<?php

namespace OzzModz\WebsiteVerification\Service\Website;

use XF\Service\ValidateAndSavableTrait;

class Verify extends \XF\Service\AbstractService
{
	use ValidateAndSavableTrait;

	/**
	 * @var \OzzModz\WebsiteVerification\Entity\Website
	 */
	protected $website;

	protected $xfValidationToken;

	public function __construct(\XF\App $app, \OzzModz\WebsiteVerification\Entity\Website $website)
	{
		parent::__construct($app);
		$this->website = $website;
	}

	public function setXenforoValidationToken($validationToken)
	{
		$this->xfValidationToken = $validationToken;
		$this->website->xenforo_validation_token = $validationToken;
	}

	protected function checkXenforoLicense()
	{
		$website = $this->website;

		/** @var \OzzModz\WebsiteVerification\Helper\XenforoLicenseApi $licenseHelper */
		$licenseHelper = $this->app->helper('OzzModz\WebsiteVerification:XenforoLicenseApi');
		$licenseInfo = $licenseHelper->getLicenseInfo($this->xfValidationToken, $website->domain);

		$error = $licenseHelper->getError();
		if ($error)
		{
			$website->error($error, 'xf_license');
			return null;
		}

		/** @var \OzzModz\WebsiteVerification\Entity\XenforoLicense $license */
		$license = $this->em()->find('OzzModz\WebsiteVerification:XenforoLicense', $licenseInfo['validation_token']);
		if (!$license)
		{
			/** @var \OzzModz\WebsiteVerification\Entity\XenforoLicense $license */
			$license = $this->em()->create('OzzModz\WebsiteVerification:XenforoLicense');
		}

		$license->setFromApiResponse($licenseInfo);

		try
		{
			$license->save();
		}
		catch (\XF\Db\DuplicateKeyException $e)
		{

		}

		if ($this->app->options()->ozzmodz_wv_xfLicenseDomainMatch && !$license->domain_match)
		{
			$website->error(\XF::phrase('ozzmodz_wv_xf_license_domain_does_not_match'), 'xf_license');
			return null;
		}

		$website->xenforo_validation_token = $this->xfValidationToken;

		return $license;
	}

	protected function checkDnsRecord()
	{
		$website = $this->website;

		/** @var \OzzModz\WebsiteVerification\Helper\Dns $dnsHelper */
		$dnsHelper = $this->app->helper('OzzModz\WebsiteVerification:Dns');
		$found = $dnsHelper->findTxtRecord($website->domain, $website->getTxtRecord());
		if (!$found)
		{
			$this->website->error(\XF::phrase('ozzmodz_vw_dns_record_not_found'), 'dns');
			return false;
		}

		$this->website->dns_verified = true;
		return $found;
	}

	protected function checkTxtFile()
	{
		$website = $this->website;
		$filePath = $website->getVerificationFilePath();

		/** @var \OzzModz\WebsiteVerification\Helper\Http $httpHelper */
		$httpHelper = $this->app->helper('OzzModz\WebsiteVerification:Http');
		$contents = $httpHelper->getFileContents('http://' . $filePath, $error)
			|| $httpHelper->getFileContents('https://' . $filePath, $error);

		if ($contents == $website->verification_code)
		{

			$this->website->file_verified = true;
			return true;
		}

		$this->website->error($error, 'file');
		return false;
	}

	protected function finalSetup()
	{
		$this->website->validation_date = \XF::$time;
	}

	protected function _validate()
	{
		$errors = [];
		$website = $this->website;

		$licenseVerified = false;
		if ($this->xfValidationToken)
		{
			if ($this->app->options()->ozzmodz_wv_xfLicenseMultiple)
			{
				$tokenUses = $this->finder('OzzModz\WebsiteVerification:Website')
					->where('xenforo_validation_token', '=', $this->xfValidationToken)
					->where('website_id', '!=', $website->website_id)
					->total();

				if ($tokenUses)
				{
					$errors[] = \XF::phrase('ozzmodz_wv_xenforo_license_token_is_already_used');
					return $errors;
				}
			}

			$licenseVerified = (bool) $this->checkXenforoLicense();
		}

		$dnsVerified = $this->checkDnsRecord();
		$fileVerified = $this->checkTxtFile();

		$verified = $licenseVerified || $dnsVerified || $fileVerified;

		$requiredVerifications = $this->app->options()->ozzmodz_wv_requiredVerifications;
		if ($requiredVerifications['xf_license'] && !$licenseVerified)
		{
			$verified = false;
		}
		if ($requiredVerifications['dns'] && !$dnsVerified)
		{
			$verified = false;
		}
		if ($requiredVerifications['file'] && !$fileVerified)
		{
			$verified = false;
		}

		$this->finalSetup();
		$website->preSave();

		if ($verified)
		{
			$website->resetErrors(['xf_license', 'dns', 'file']);
		}

		$errors += $website->getErrors();
		return $errors;
	}

	protected function _save()
	{
		$website = $this->website;


		$website->save();

		return $website;
	}
}