<?php

namespace OzzModz\WebsiteVerification\XF\Api\Controller;

use XF\Mvc\Entity\Entity;
use XF\Mvc\ParameterBag;

class User extends XFCP_User
{
	/**
	 * @api-desc Gets a page of websites on the specified user.
	 *
	 * @api-in int $page
	 *
	 * @api-see self::getProfilePostsForUserPaginated
	 *
	 * @param ParameterBag $params
	 * @return \XF\Api\Mvc\Reply\ApiResult|\XF\Mvc\Reply\AbstractReply
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionGetVerificationWebsites(ParameterBag $params)
	{
		$this->assertApiScope('ozzmodz_wv_website:read');

		/** @var \OzzModz\WebsiteVerification\XF\Entity\User $user */
		$user = $this->assertViewableUser($params->user_id);

		if (\XF::isApiCheckingPermissions() && !$user->canViewWebsiteVerificationDomains($error))
		{
			return $this->noPermission($error);
		}

		$postData = $this->getVerificationWebsitesForUserPaginated($user, $this->filterPage());

		return $this->apiResult($postData);
	}

	/**
	 * @api-out OzzModz_WebsiteVerification_Website[] $websites List of websites on the requested page
	 * @api-out pagination $pagination Pagination details
	 *
	 * @param \XF\Entity\User $user
	 * @param int $page
	 * @param null|int $perPage
	 *
	 * @return array
	 *
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function getVerificationWebsitesForUserPaginated(\XF\Entity\User $user, int $page = 1, int $perPage = null)
	{
		$perPage = intval($perPage);
		if ($perPage <= 0)
		{
			$perPage = 20;
		}

		$finder = $this->setupVerificationWebsiteFinder($user);

		$total = $finder->total();

		$this->assertValidApiPage($page, $perPage, $total);

		$websites = $finder->limitByPage($page, $perPage)->fetch();

		$results = $websites->toApiResults(Entity::VERBOSITY_NORMAL, [
			'with_xenforo_license' => true
		]);

		return [
			'websites' => $results,
			'pagination' => $this->getPaginationData($results, $page, $perPage, $total)
		];
	}

	/**
	 * @param \XF\Entity\User $user
	 * @return \OzzModz\WebsiteVerification\Finder\Website
	 */
	protected function setupVerificationWebsiteFinder(\XF\Entity\User $user)
	{
		/** @var \OzzModz\WebsiteVerification\Finder\Website $finder */
		$finder = $this->finder('OzzModz\WebsiteVerification:Website');
		$finder->byUser($user)
			->order('date_added', 'DESC')
			->with('api');

		return $finder;
	}

}